from baseDatamodel import *
from theia.aws.containers.eks import *
from baseDefaultInputs import defaultBaseInputs
from theia import Theia
from theia.aws.network import Network, NetworkOptions, NetworkSubnet
from typing import Any


class pcpNetworkSubnet:

    def __init__(self, 
                 userInput: baseVpcSubnets):
        
        self.cidr = userInput.cidr_block
        self.type = userInput.type
        self.az = userInput.az


    def theiaNetworkSubnet(self):
        subnet = NetworkSubnet(cidr_block=self.cidr,
                               type=self.type,
                               availability_zone=self.az)
        return subnet

class pcpNetwork:

    @staticmethod
    def getpcpVpcSubnets(userInputs: baseValidator,
                         defaultVpc: bool,
                         defaultInputs: dict = defaultBaseInputs):

        networkVpcSubnets = []
        if defaultVpc:
            defaultSubnets = defaultInputs['Network']['vpc_subnets']
            for subnet in defaultSubnets:
                vpcSubnet = pcpNetworkSubnet(baseVpcSubnets(**subnet))
                networkVpcSubnets.append(vpcSubnet)
        else:
            userSubnets = userInputs.Modules.base.Network.subnet_cidr
            for subnet in userSubnets:
                vpcSubnet = pcpNetworkSubnet(subnet)
                networkVpcSubnets.append(vpcSubnet)
        return networkVpcSubnets
    
    def __init__(self,
                 userInputs: baseValidator,
                 defaultInputs: dict = defaultBaseInputs):

        self.defaultVpc = True

        self.networkName = base.getpcpBaseResourceName(userInput=userInputs)['NetworkName']
        self.vpc_cidr = defaultInputs['Network']['vpc_cidr']


        if userInputs.Modules.base.Network.vpc_cidr:
            self.defaultVpc = False
            self.vpc_cidr = userInputs.Modules.base.Network.vpc_cidr

        self.vpc_subnets = pcpNetwork.getpcpVpcSubnets(userInputs=userInputs, defaultVpc=self.defaultVpc)

        self.creteNatGateway = userInputs.Modules.base.Network.create_nat_gateway
        self.creteIGW = userInputs.Modules.base.Network.create_internet_gateway

class eksBaseNodeBlockDeviceArgs:

    def __init__(self,
                 blockDeviceArgs: baseEksNodeBlockDeviceArgs):
        
        self.ebs_iops = blockDeviceArgs.ebs_iops
        self.volume_type = blockDeviceArgs.volume_type
        self.volume_size = blockDeviceArgs.volume_size

class eksBaseNodeGroupTaint:

    def __init__(self,
                 taint: baseEksNodeGroupTaint):
        
        self.effect = taint.effect
        self.key = taint.key
        self.value = taint.value

class eksBaseNodeGroup:
    
    def __init__(self,
                 eksBaseNodeGroup: baseEksNodeGroup):
        
        self.name = eksBaseNodeGroup.name
        self.zones = eksBaseNodeGroup.zones
        self.spot_instance = eksBaseNodeGroup.spot_instance
        self.instance_type = eksBaseNodeGroup.instance_types

        self.taints = []
        for taint in eksBaseNodeGroup.taints:
            newTaint = eksBaseNodeGroupTaint(taint)
            self.taints.append(newTaint)

        self.block_device = []
        for arg in eksBaseNodeGroup.block_device_args:
            newBlockDeviceArg = eksBaseNodeBlockDeviceArgs(arg)
            self.block_device.append(newBlockDeviceArg)

        self.scalingConfig = eksBaseNodeGroup.scaling_config
        self.userdata = eksBaseNodeGroup.user_data

    def getTheiaScalingConfig(self):

        theiaScalingConfig = ScalingConfig(desired_size=self.scalingConfig.desired_size,
                                           max_size=self.scalingConfig.max_size,
                                           min_size=self.scalingConfig.min_size)
        return theiaScalingConfig


def getTheiaBlockDeviceArgs(args: List[eksBaseNodeBlockDeviceArgs]):
    theiaArgs = []
    for arg in args:
        newTheiaArg = BlockDeviceArgs(
            device_name = baseEksNodeBlockDeviceArgs().name,
            ebs_iops = arg.ebs_iops,
            volume_size = arg.volume_size,
            volume_type = arg.volume_type
        )
        theiaArgs.append(newTheiaArg)
    return theiaArgs

def getTheiaTaints(taints: List[eksBaseNodeGroupTaint]):
    theiaTaints = []
    for taint in taints:
        newTaint = NodeGroupTaint(
            effect = taint.effect,
            key = taint.key,
            value = taint.value
        )
        theiaTaints.append(newTaint)
    return theiaTaints

def getTheiaNodeGroups(nodeGroups: List[eksBaseNodeGroup],
                       baseNetwork: Network,
                       nodeRole: Role):
    
    NodeGroups = []
    for nodeGroup in nodeGroups:
        theiaNodeGroup = EksNodeGroup(
            name = nodeGroup.name,
            zones = nodeGroup.zones,
            spot_instance = nodeGroup.spot_instance,
            instance_types = nodeGroup.instance_type,
            taints = getTheiaTaints(nodeGroup.taints),
            node_role_arn = nodeRole.arn,
            block_device_args= getTheiaBlockDeviceArgs(nodeGroup.block_device),
            scaling_config = nodeGroup.getTheiaScalingConfig(),
            security_group_ids = [baseNetwork.sg.id],
            labels={"role" : f"{nodeGroup.name}"}
            )
        NodeGroups.append(theiaNodeGroup)
    return NodeGroups



class eksBaseSettings:
    
    def __init__(self,
                 eksSettings : baseEksSettings):
        
        self.public_acces_cidr = eksSettings.public_acces_cidr
        self.default_addons_to_remove = eksSettings.default_addons_to_remove
        self.subnet_zones = eksSettings.subnet_zones
        nodeGroups = eksSettings.nodegroups

        self.nodeGroups = []
        for nodeGroup in nodeGroups:
            newNodeGroup = eksBaseNodeGroup(nodeGroup)
            self.nodeGroups.append(newNodeGroup)

    def getTheiaSettings(self,
                         theiaNetwork: Network,
                         clusterRole: Role,
                         nodeRole: Role):
        
        # print(self.public_acces_cidr)
        settings = EksSettings(
            override_eks_role = clusterRole,
            subnet_zones = self.subnet_zones,
            public_access_cidrs = [f"{self.public_acces_cidr}"],
            default_addons_to_remove = self.default_addons_to_remove,
            nodegroups = getTheiaNodeGroups(nodeGroups=self.nodeGroups,
                                            baseNetwork=theiaNetwork,
                                            nodeRole=nodeRole)
        )
        return settings

class eksBase:

    def __init__(self,
                 userInputs):
        
        eksInputs = userInputs.Modules.base.Eks
        
        self.clusterName = base.getpcpBaseResourceName(userInput=userInputs)['EksCluster']
        self.eks_version = eksInputs.eks_version
        self.deployment_type = eksInputs.deployment_type
        self.create_kms_key = eksInputs.create_kms_key
        self.settings = eksBaseSettings(eksInputs.settings)

    def getTheiaEks(self, 
                    userInputs: baseValidator,
                    theiaNetwork: Network,
                    basicTags: dict):
    
        CLUSTER_MANAGED_POLICIES = [
        "arn:aws:iam::aws:policy/AmazonEKSClusterPolicy",
        "arn:aws:iam::aws:policy/AmazonEKSServicePolicy",
        ]
        clusterInlinePolicy = json.dumps(
            {
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Sid": "VisualEditor0",
                    "Effect": "Allow",
                    "Action": [
                        "s3:GetObject",
                        "s3:ListBucket",
                        "s3:PutObject"
                    ],
                    "Resource": [
                        "arn:aws:s3:::core-service-iota",
                        "arn:aws:s3:::core-service-iota/*"
                    ]
                },
                {
                    "Sid": "",
                    "Effect": "Allow",
                    "Action": [
                        "kms:Encrypt",
                        "kms:Decrypt",
                        "kms:GenerateDataKey"
                    ],
                    "Resource": [
                        "arn:aws:kms:us-east-1:109667701036:key/a4a47da3-56c9-4d41-9757-5d2415f11bcc"
                    ]
                }
            ]
        })

        inlinePolicy = InlinePolicy(name="clusterinlinePolocy",policy=clusterInlinePolicy)
        
        clusterRole = Role(
            resource_name = f"{eksBase.__name__}-cluster-role",
            assume_role_policy=json.dumps(
                {
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Action": "sts:AssumeRole",
                            "Principal": {"Service": "ec2.amazonaws.com"},
                            "Effect": "Allow",
                            "Sid": "",
                        }
                    ],
                }
            ),
            managed_policy_arns = CLUSTER_MANAGED_POLICIES,
            inline_policies=[inlinePolicy]
                ) 
        NODEGROUP_MANAGED_POLICY_ARNS = [
        "arn:aws:iam::aws:policy/AmazonEKSWorkerNodePolicy",
        "arn:aws:iam::aws:policy/AmazonEKS_CNI_Policy",
        "arn:aws:iam::aws:policy/AmazonEC2ContainerRegistryReadOnly",
        "arn:aws:iam::aws:policy/CloudWatchAgentServerPolicy",
        "arn:aws:iam::aws:policy/AmazonSSMManagedInstanceCore",
        ]
        
        NodeRole = Role(
            f"{eksBase.__name__}-node-roles",
            assume_role_policy=json.dumps(
                {
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Action": "sts:AssumeRole",
                            "Principal": {"Service": "ec2.amazonaws.com"},
                            "Effect": "Allow",
                            "Sid": ""
                        }
                    ],
                }
            ),
            managed_policy_arns=NODEGROUP_MANAGED_POLICY_ARNS,
        )

        auth_data = [
            AuthConfigMapData(
                rolearn=NodeRole.arn,
                username="system:node:{{{{EC2PrivateDNSName}}}}",
                groups=["system:bootstrappers", "system:nodes", "system:master"],
            ),
            AuthConfigMapData(
                rolearn=clusterRole.arn,
                username="system:node:{{{{EC2PrivateDNSName}}}}",
                groups=["system:master"],
            )
        ]

        eks = Eks(
                resource_name = base.getpcpBaseResourceName(userInput=userInputs)['EksCluster'],
                eks_version = self.eks_version,
                subnet_ids = theiaNetwork.subnets_per_zone[f"{self.deployment_type}"],
                deployment_type = self.deployment_type,
                create_kms_key = self.create_kms_key,
                security_group_ids=[theiaNetwork.sg.id],
                settings= self.settings.getTheiaSettings(theiaNetwork=theiaNetwork, clusterRole=clusterRole, nodeRole=NodeRole),
                auth_data=auth_data,
                opts = ResourceOptions(depends_on=[theiaNetwork]),
                tags=basicTags
                )
        return eks


class base:
    @staticmethod
    def getpcpBaseStackName(userInputs: baseValidator):
        return f"{userInputs.project}-{userInputs.env}-{base.__name__}"
    
    @staticmethod
    def getpcpBaseResourceName(userInput: baseValidator):
        resourceList = {}
        baseStack = base.getpcpBaseStackName(userInputs=userInput)
        eks = {
            "EksCluster" : f"{baseStack}-eks"
        }
        network = {
            "NetworkName" : f"{baseStack}-network"
        }
        resourceList.update(network)
        resourceList.update(eks)
        return resourceList

    def __init__(self,
                 userInputs: dict,
                 defaultInputs = defaultBaseInputs):
        
        self.userdata = baseValidator(**userInputs)

        self.project = f"{self.userdata.project}-{self.userdata.env}"
        self.email = self.userdata.email
        self.backend_url = self.userdata.backend_url
        self.region = self.userdata.region
        self.stack = base.__name__

        self.defaultInputs = defaultInputs

        self.baseInputs = baseResources(**userInputs["Modules"][f"{base.__name__}"])

        self.defaultInputs['tags']["Stack"] = base.getpcpBaseStackName(self.userdata)
        self.defaultInputs['tags']['created_by'] = self.email

        self.tags = self.defaultInputs['tags']

        self.eksInputs = eksBase(userInputs=self.userdata)


    def infra(self):

        def program():
                
            networkInfra = pcpNetwork(userInputs=self.userdata)

            # eksInfra = self.eksInputs.getTheiaEks()

            
            networkSpecs = []
            for subnet in networkInfra.vpc_subnets:
                theiaSubnet = subnet.theiaNetworkSubnet()
                networkSpecs.append(theiaSubnet)

            networkOpts = NetworkOptions(create_nat_gateway=networkInfra.creteNatGateway,
                                        create_internet_gateway=networkInfra.creteIGW,
                                        create_vpc_security_group=True)
            
            theiaNetworkInfra = Network(resource_name=base.getpcpBaseResourceName(self.userdata)['NetworkName'],
                                            vpc_cidr=networkInfra.vpc_cidr,
                                            subnet_specs=networkSpecs,
                                            tenancy="default",
                                            tags = self.tags,
                                            create_opts=networkOpts)
            
            theiaEksInfra = self.eksInputs.getTheiaEks(theiaNetwork=theiaNetworkInfra,userInputs=self.userdata, basicTags=self.tags)

            # print(theiaEksInfra)

        t =  Theia(project=self.project,
                    stack=self.stack,
                    program=program,
                    region=self.region,
                    backend_url=self.backend_url)
        
        return t
    
    def __call__(self, command: Literal['preview', 'up', 'destroy', 'refresh']) -> Any:
        t = self.infra()
        getattr(t, command)()






# userInputs = {
#         "project" : "test3",
#         "env" : "dev",
#         "email" : "ayush.kumar@gmail.com",
#         "region" : "ap-south-1",
#         "backend_url" : "s3://theia-testing-ayush",
#         "Modules" : {
#             "base" : {
#                 "Network": {
#                     "vpc_cidr": "10.0.0.0/16",
#                     "subnet_cidr": [
#                         {
#                             "cidr_block": "10.0.2.0/24",
#                             "type": "public",
#                             "az": "ap-south-1a"
#                         },
#                         {
#                             "cidr_block": "10.0.3.0/24",
#                             "type": "public",
#                             "az": "ap-south-1b"
#                         },
#                         {
#                             "cidr_block": "10.0.1.0/24",
#                             "type": "private",
#                             "az": "ap-south-1b"
#                         }
#                     ],
#                     "create_internet_gateway": True,
#                     "create_nat_gateway": "none"
#                 },
#                 "Eks" : {
#                     "eks_version" : "1.24",
#                     "deployment_type" : "public",
#                     "create_kms_key" : False,
#                     "settings" : {
#                         "public_acces_cidr" : "0.0.0.0/0",
#                         "subnet_zones" : ["ap-south-1a", "ap-south-1b"],
#                         "nodegroups" : [
#                             {
#                                 "name" : "mgmt",
#                                 "zones" : ["ap-south-1a"],
#                                 "spot_instance" : True
#                             }
#                         ]
#                     }
#                 }
#             }
#         }
#     }

# test = {
#     "Modules" : {
#         "base" : {
#             "Eks" : {
#                 "eks_version" : 1.23,
#                 "deployment_type" : "public",
#                 "create_kms_key" : False
#             }
#         }
#     }
# }

# t = baseValidator(**test)
# print(t.Modules.base.Eks.eks_version)
# t = base(userInputs=userInputs)
# t('up')
# t.infra()
# print(t.infra())
# print(t.eksInputs.eks_version)
# t('destroy')
# input('Press enter to continue ')
# t('destroy')
# t2 = pcpNetwork(userInputs=(baseValidator(**userInputs)))
# t('destroy')
# print(t2)
