from typing import Optional, Dict, Literal, List, Mapping
from pydantic import BaseModel, validator

class baseEksScalingConfig(BaseModel):
    desired_size: int = 5
    max_size: int = 10
    min_size: int = 2

class baseEksNodeBlockDeviceArgs(BaseModel):
    name: str = "/dev/xvda"
    ebs_iops: int = 50
    volume_type: Literal["standard", "gp2", "gp3", "io1", "io2", "sc1", "st1"] = "gp3"
    volume_size: int = 50

class baseEksNodeGroupTaint(BaseModel):
    effect: Optional[Literal["NO_SCHEDULE", "NO_EXECUTE", "PREFER_NO_SCHEDULE"]] = "NO_SCHEDULE"
    key: Optional[str] = "mgmt"
    value: Optional[str] = "true"

class baseEksNodeGroup(BaseModel):
    name: Optional[str] = "mgmt"
    zones: List[str]
    spot_instance: bool
    instance_types: Optional[List[str]] = ["t3a.xlarge"]
    taints: List[baseEksNodeGroupTaint] = [baseEksNodeGroupTaint()]
    block_device_args: List[baseEksNodeBlockDeviceArgs] = [baseEksNodeBlockDeviceArgs()]
    scaling_config: baseEksScalingConfig = baseEksScalingConfig()
    user_data: Optional[str] = None

class baseEksSettings(BaseModel):
    public_acces_cidr: str
    default_addons_to_remove: Optional[List[str]] = None
    subnet_zones: Optional[List[str]] = None
    nodegroups: List[baseEksNodeGroup]

class baseEksValidator(BaseModel):
    eks_version: str
    deployment_type: Literal['public', 'private']
    create_kms_key: bool
    settings: baseEksSettings

class baseVpcSubnets(BaseModel):
    cidr_block: str
    type: Literal["public", "private"]
    az: str

class basenetworkValidator(BaseModel):
    vpc_cidr: str
    subnet_cidr: List[baseVpcSubnets]
    create_internet_gateway: bool
    create_nat_gateway: Literal['none', 'single', 'per az']

    # @validator('vpc_cidr', 'subnet_cidr')
    # def subnetValidator(cls,
    #                     value,
    #                     values):
    #     if value is not None and values == []:
    #         raise ValueError('Provide subnet cidr')
    #     elif value is None and values != []:
    #         raise ValueError('Provide vpc cidr')


class baseResources(BaseModel):
    Network : basenetworkValidator
    Eks: baseEksValidator


class modules(BaseModel):
    base: baseResources

class baseValidator(BaseModel):
    project: str
    env: str
    email: str
    region: str
    backend_url: str
    Modules: modules